from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from monobank._core.client.acquiring import AcquiringAPIClient


class WalletFacade:
    def __init__(self, client: AcquiringAPIClient):
        self.client = client

    def delete_card(self, params=None, data=None):
        """
        Видалення картки

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.delete("merchant/wallet/card", params=params, data=data)

    def payment(self, params=None, data=None):
        """
        Створення платежу за токеном картки

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/wallet/payment", params=params, data=data)

    def __call__(self, params=None, data=None):
        """
        Список карток у гаманці

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/wallet", params=params, data=data)



from urllib import request, parse
from urllib.parse import urljoin
import json


class BaseAPIClient:
    def __init__(self, base_url, response_serializer=BaseAPIClient.get_json):
        """
        Base API client

        :param base_url: request base url to API
        :param response_serializer: method that accepts urllib.response
        """
        self.base_url = base_url
        self._response_serializer = response_serializer
        self._headers = {'Content-Type': 'application/json'}  # Default header

    @property
    def headers(self):
        return self._headers

    @headers.setter
    def headers(self, value):
        self._headers = value

    def _request(self, method, path, params=None, data=None):
        url = urljoin(self.base_url, path)

        if params:
            query_string = parse.urlencode(params)
            url += '?' + query_string

        if data:
            data_bytes = json.dumps(data).encode('utf-8')
        else:
            data_bytes = None

        req = request.Request(url, data=data_bytes, method=method)

        for key, value in self.headers.items():
            req.add_header(key, value)

        with request.urlopen(req) as response:
            return self._response_serializer(response)

    def get(self, path, params=None, data=None):
        return self._request('GET', path, params=params, data=data)

    def post(self, path, params=None, data=None):
        return self._request('POST', path, params=params, data=data)

    def put(self, path, params=None, data=None):
        return self._request('PUT', path, params=params, data=data)

    def delete(self, path, params=None, data=None):
        return self._request('DELETE', path, params=params, data=data)

    @staticmethod
    def get_json(response):
        """
        Get json from response

        :param response: urllib.response
        :return: JSON to PON (Python Object Notation)
        """
        return json.loads(response.read().decode('utf-8'))



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from monobank._core.client.acquiring import AcquiringAPIClient


class InvoiceFacade:
    def __init__(self, client: AcquiringAPIClient):
        self.client = client

    def create(self, params=None, data=None):
        """
        Створення рахунку для оплати

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/invoice/create", params=params, data=data)

    def status(self, params=None, data=None):
        """
        Метод перевірки статусу рахунку при розсинхронізації з боку продавця
        або відсутності webHookUrl при створенні рахунку.

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/invoice/status", params=params, data=data)

    def cancel(self, params=None, data=None):
        """
        Скасування успішної оплати рахунку

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/invoice/cancel", params=params, data=data)

    def remove(self, params=None, data=None):
        """
        Інвалідація рахунку, якщо за ним ще не було здіснено оплати

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/invoice/remove", params=params, data=data)

    def payment_info(self, params=None, data=None):
        """
        Дані про успішну оплату, якщо вона була здійснена

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/invoice/payment-info", params=params, data=data)

    def finalize(self, params=None, data=None):
        """
        Фіналізація суми холду

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/invoice/finalize", params=params, data=data)

    def payment_direction(self, params=None, data=None):
        """
        Створення рахунку та його оплата за реквізитами картки.
        Увага, це апі буде працювати тільки за умови наявності у мерчанта активного PCI DSS сертифіката!

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/invoice/payment-direction", params=params, data=data)

    def fiscal_checks(self, params=None, data=None):
        """
        Метод для отримання даних фіскальних чеків та їх статусів

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/invoice/fiscal-checks", params=params, data=data)



from monobank._core.client.base import BaseAPIClient


class AcquiringAPIClient(BaseAPIClient):
    def __init__(self, token: str, response_serializer=BaseAPIClient.get_json):
        super().__init__(base_url="https://api.monobank.ua/api/", response_serializer=response_serializer)
        self._token = token
        self.headers |= {'X-Token': self._token}



from monobank.api._acquiring._merchant import MerchantFacade
from monobank._core.client.acquiring import AcquiringAPIClient


class AcquiringAPI:
    def __init__(self, api_key, response_serializer=AcquiringAPIClient.get_json):
        self._api_key = api_key
        self._client = AcquiringAPIClient(token=self._api_key, response_serializer=response_serializer)
        self.merchant = MerchantFacade(client=self._client)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from monobank._core.client.acquiring import AcquiringAPIClient


class QRFacade:
    def __init__(self, client: AcquiringAPIClient):
        self.client = client

    def details(self, params=None, data=None):
        """
        Інформація про QR-касу, лише для активованих QR-кас

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/qr/details", params=params, data=data)

    def reset_amount(self, params=None, data=None):
        """
        Видалення суми оплати з QR-каси

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.post("merchant/qr/reset-amount", params=params, data=data)

    def list(self, params=None, data=None):
        """
        Список QR-кас

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/qr/list", params=params, data=data)


from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from monobank._core.client.acquiring import AcquiringAPIClient

from monobank.api._acquiring._merchant.details import DetailsFacade
from monobank.api._acquiring._merchant.invoice import InvoiceFacade
from monobank.api._acquiring._merchant.pubkey import PubkeyFacade
from monobank.api._acquiring._merchant.qr import QRFacade
from monobank.api._acquiring._merchant.statement import StatementFacade
from monobank.api._acquiring._merchant.wallet import WalletFacade


class MerchantFacade:
    def __init__(self, client: AcquiringAPIClient):
        self.client = client
        self.invoice = InvoiceFacade(client=client)
        self.details = DetailsFacade(client=client)
        self.pubkey = PubkeyFacade(client=client)
        self.qr = QRFacade(client=client)
        self.statement = StatementFacade(client=client)
        self.wallet = WalletFacade(client=client)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from monobank._core.client.acquiring import AcquiringAPIClient


class SubmerchantFacade:
    def __init__(self, client: AcquiringAPIClient):
        self.client = client

    def list(self, params=None, data=None):
        """
        Дане апі потрібне обмеженому колу осіб, яким при створенні рахунку треба явно вказувати термінал

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/submerchant/list", params=params, data=data)



from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from monobank._core.client.acquiring import AcquiringAPIClient


class PubkeyFacade:
    def __init__(self, client: AcquiringAPIClient):
        self.client = client

    def __call__(self, params=None, data=None):
        """
        Отримання відкритого ключа для перевірки підпису, який включено у вебхуки.
        Ключ можна кешувати і робити запит на отримання нового,
        коли верифікація підпису з поточним ключем перестане працювати.
        Кожного разу робити запит на отримання ключа не треба

        :param params: параметри запиту
        :param data: дані запиту
        :return: Serialized response (default get_json)
        """
        return self.client.get("merchant/pubkey", params=params, data=data)
